@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright 2021 Ramil Nugmanov <nougmanoff@protonmail.com>
+#  Copyright 2021, 2022 Ramil Nugmanov <nougmanoff@protonmail.com>
 #  This file is part of chython.
 #
 #  chython is free software; you can redistribute it and/or modify
@@ -30,15 +30,17 @@ if TYPE_CHECKING:
 class Kekule:
     __slots__ = ()
 
-    def kekule(self: Union['Kekule', 'MoleculeContainer']) -> bool:
+    def kekule(self: Union['Kekule', 'MoleculeContainer'], *, buffer_size=7) -> bool:
         """
         Convert structure to kekule form. Return True if found any aromatic ring. Set implicit hydrogen count and
         hybridization marks on atoms.
 
         Only one of possible double/single bonds positions will be set.
         For enumerate bonds positions use `enumerate_kekule`.
+
+        :param buffer_size: number of attempts of pyridine form searching.
         """
-        kekule = next(self.__kekule_full(), None)
+        kekule = next(self.__kekule_full(buffer_size), None)
         if kekule:
             bonds = self._bonds
             atoms = set()
@@ -56,7 +58,7 @@ class Kekule:
         """
         Enumerate all possible kekule forms of molecule.
         """
-        for form in self.__kekule_full():
+        for form in self.__kekule_full(0):
             copy = self.copy()
             bonds = copy._bonds
             atoms = set()
@@ -70,6 +72,7 @@ class Kekule:
 
     def __fix_rings(self: 'MoleculeContainer'):
         bonds = self._bonds
+        charges = self._charges
         seen = set()
         for q, af, bf in rules:
             for mapping in q.get_mapping(self, automorphism_filter=False):
@@ -78,10 +81,9 @@ class Kekule:
                     continue
                 seen.update(match)
 
-                for n, fix in af.items():
+                for n, c in af.items():
                     n = mapping[n]
-                    for key, value in fix.items():
-                        getattr(self, key)[n] = value
+                    charges[n] = c
                 for n, m, b in bf:
                     n = mapping[n]
                     m = mapping[m]
@@ -98,7 +100,7 @@ class Kekule:
         neighbors = self.neighbors
 
         rings = defaultdict(list)  # aromatic skeleton
-        pyroles = set()
+        pyrroles = set()
 
         double_bonded = defaultdict(list)
         triple_bonded = set()
@@ -113,7 +115,7 @@ class Kekule:
                     triple_bonded.add(n)
 
         if not rings:
-            return rings, pyroles, set()
+            return rings, pyrroles, set()
         elif not triple_bonded.isdisjoint(rings):
             raise InvalidAromaticRing('triple bonds connected to rings')
 
@@ -184,34 +186,34 @@ class Kekule:
                             raise InvalidAromaticRing
                     elif ab == 3:
                         double_bonded.add(n)
-                    elif ab == 2:  # benzene an[cat]ion or pyrole
-                        pyroles.add(n)
+                    elif ab == 2:  # benzene an[cat]ion or pyrrole
+                        pyrroles.add(n)
                     else:
                         raise InvalidAromaticRing
                 else:
                     raise InvalidAromaticRing
             elif an in (7, 15, 33):
-                if ac == 0:  # pyrole or pyridine. include radical pyrole
+                if ac == 0:  # pyrrole or pyridine. include radical pyrrole
                     if radicals[n]:
                         if ab != 2:
                             raise InvalidAromaticRing
                         double_bonded.add(n)
                     elif ab == 3:
-                        if an == 7:  # pyrole only possible
+                        if an == 7:  # pyrrole only possible
                             double_bonded.add(n)
                         else:  # P(III) or P(V)H
-                            pyroles.add(n)
+                            pyrroles.add(n)
                     elif ab == 2:
                         ah = hydrogens[n]
                         if ah is None:
-                            pyroles.add(n)
-                        elif ah == 1:  # only pyrole
+                            pyrroles.add(n)
+                        elif ah == 1:  # only pyrrole
                             double_bonded.add(n)
                         elif ah:
                             raise InvalidAromaticRing
                     elif ab != 4 or an not in (15, 33):  # P(V) in ring
                         raise InvalidAromaticRing
-                elif ac == -1:  # pyrole only
+                elif ac == -1:  # pyrrole only
                     if ab != 2 or radicals[n]:
                         raise InvalidAromaticRing
                     double_bonded.add(n)
@@ -220,8 +222,8 @@ class Kekule:
                 elif radicals[n]:
                     if ab != 2:  # not cation-radical pyridine
                         raise InvalidAromaticRing
-                elif ab == 2:  # pyrole cation or protonated pyridine
-                    pyroles.add(n)
+                elif ab == 2:  # pyrrole cation or protonated pyridine
+                    pyrroles.add(n)
                 elif ab != 3:  # not pyridine oxyde
                     raise InvalidAromaticRing
             elif an == 8:  # furan
@@ -269,7 +271,7 @@ class Kekule:
                         else:
                             ah = hydrogens[n]
                             if ah is None:  # b1ccccc1, C=1OBOC=1 or B1C=CC=N1
-                                pyroles.add(n)
+                                pyrroles.add(n)
                             elif ah == 1:  # C=1O[BH]OC=1 or [BH]1C=CC=N1
                                 double_bonded.add(n)
                             elif ah:
@@ -286,21 +288,21 @@ class Kekule:
                 elif ac == -1:
                     if ab == 2:
                         if not radicals[n]:  # C=1O[B-]OC=1 or [bH-]1ccccc1
-                            pyroles.add(n)
+                            pyrroles.add(n)
                         # anion-radical is benzene like
                     elif radicals[n]:  # C=1O[B-*](R)OC=1
                         double_bonded.add(n)
                     else:
-                        pyroles.add(n)
+                        pyrroles.add(n)
                 else:
                     raise InvalidAromaticRing
             else:
                 raise InvalidAromaticRing(f'only B, C, N, P, O, S, Se, Te possible, not: {atoms[n].atomic_symbol}')
-        return rings, pyroles, double_bonded
+        return rings, pyrroles, double_bonded
 
-    def __kekule_full(self):
+    def __kekule_full(self, buffer_size):
         self.__fix_rings()  # fix bad aromatic rings
-        rings, pyroles, double_bonded = self.__prepare_rings()
+        rings, pyrroles, double_bonded = self.__prepare_rings()
         atoms = set(rings)
         components = []
         while atoms:
@@ -317,21 +319,21 @@ class Kekule:
             components.append(component)
             atoms.difference_update(component)
 
-        for keks in lazy_product(*(_kekule_component(c, double_bonded & c.keys(), pyroles & c.keys())
+        for keks in lazy_product(*(_kekule_component(c, double_bonded & c.keys(), pyrroles & c.keys(), buffer_size)
                                    for c in components)):
             yield [x for x in keks for x in x]
 
 
-def _kekule_component(rings, double_bonded, pyroles):
+def _kekule_component(rings, double_bonded, pyrroles, buffer_size):
     # (current atom, previous atom, bond between cp atoms, path deep for cutting [None if cut impossible])
     stack: List[List[Tuple[int, int, int, Optional[int]]]]
     if double_bonded:  # start from double bonded if exists
         start = next(iter(double_bonded))
         stack = [[(next(iter(rings[start])), start, 1, 0)]]
-    else:  # select not pyrole not condensed atom
+    else:  # select not pyrrole not condensed atom
         try:
-            start = next(n for n, ms in rings.items() if len(ms) == 2 and n not in pyroles)
-        except StopIteration:  # all pyroles. select not condensed atom.
+            start = next(n for n, ms in rings.items() if len(ms) == 2 and n not in pyrroles)
+        except StopIteration:  # all pyrroles. select not condensed atom.
             try:
                 start = next(n for n, ms in rings.items() if len(ms) == 2)
             except StopIteration:  # fullerene?
@@ -347,6 +349,7 @@ def _kekule_component(rings, double_bonded, pyroles):
     path = []
     hashed_path = set()
     nether_yielded = True
+    buffer = []
 
     while stack:
         atom, prev_atom, bond, _ = stack[-1].pop()
@@ -354,9 +357,31 @@ def _kekule_component(rings, double_bonded, pyroles):
         hashed_path.add(atom)
 
         if len(path) == size:
-            yield path
             if nether_yielded:
                 nether_yielded = False
+            if pyrroles and buffer_size:  # prioritize pyridine over pyrrole
+                g = defaultdict(int)
+                for n, m, b in path:
+                    g[n] += b
+                    g[m] += b
+                # should be pairs of pyrrole atoms
+                if sum(b == 2 and n in pyrroles for n, b in g.items()) >= 2:
+                    if len(buffer) == buffer_size:  # optimization. try only few times to prevent freezes.
+                        buffer_size = 0  # disable bufferization
+                        yield from buffer
+                        yield path
+                        buffer = []
+                    else:
+                        buffer.append(path)
+                else:
+                    yield path
+                    buffer_size = 0  # disable bufferization
+                    if buffer:  # empty buffer
+                        yield from buffer
+                        buffer = []
+            else:
+                yield path
+
             del stack[-1]
             if stack:
                 path = path[:stack[-1][-1][-1]]
@@ -386,8 +411,8 @@ def _kekule_component(rings, double_bonded, pyroles):
                             hashed_path = {x for x, *_ in path}
                         continue
                 elif double_bonded:  # we in quinone ring. finish should be single bonded
-                    # side-path for storing double bond or atom is quinone or pyrole
-                    if for_stack or atom in double_bonded or atom in pyroles:
+                    # side-path for storing double bond or atom is quinone or pyrrole
+                    if for_stack or atom in double_bonded or atom in pyrroles:
                         stack[-1].insert(0, (loop, atom, 1, None))
                     else:
                         del stack[-1]
@@ -405,10 +430,10 @@ def _kekule_component(rings, double_bonded, pyroles):
                     stack[-1].remove((atom, next_atom, 1, None))  # remove fork from stack
                 for next_atom in for_stack:
                     stack[-1].append((next_atom, atom, 1, None))
-            elif len(for_stack) == 1:  # easy path grow. next bond double or include single for pyroles
+            elif len(for_stack) == 1:  # easy path grow. next bond double or include single for pyrroles
                 next_atom = for_stack[0]
                 if next_atom in double_bonded:  # need double bond, but next atom quinone
-                    if atom in pyroles:
+                    if atom in pyrroles:
                         stack[-1].append((next_atom, atom, 1, None))
                     else:
                         del stack[-1]
@@ -416,7 +441,7 @@ def _kekule_component(rings, double_bonded, pyroles):
                             path = path[:stack[-1][-1][-1]]
                             hashed_path = {x for x, *_ in path}
                 else:
-                    if atom in pyroles:  # try pyrole and pyridine
+                    if atom in pyrroles:  # try pyrrole and pyridine
                         opposite = stack[-1].copy()
                         opposite.append((next_atom, atom, 2, None))
                         stack[-1].append((next_atom, atom, 1, len(path)))
@@ -448,7 +473,7 @@ def _kekule_component(rings, double_bonded, pyroles):
                     opposite.append((next_atom2, atom, 1, None))
                     opposite.append((next_atom1, atom, 2, None))
                     stack.append(opposite)
-            elif closures and atom not in pyroles:  # need double bond, but closure should be single bonded
+            elif closures and atom not in pyrroles:  # need double bond, but closure should be single bonded
                 del stack[-1]
                 if stack:
                     path = path[:stack[-1][-1][-1]]
@@ -456,6 +481,8 @@ def _kekule_component(rings, double_bonded, pyroles):
 
     if nether_yielded:
         raise InvalidAromaticRing(f'kekule form not found for: {list(rings)}')
+    elif buffer:  # optimal solution not found. return available.
+        yield from buffer
 
 
 __all__ = ['Kekule']
