@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright 2018-2021 Ramil Nugmanov <nougmanoff@protonmail.com>
+#  Copyright 2018-2022 Ramil Nugmanov <nougmanoff@protonmail.com>
 #  Copyright 2021 Dmitrij Zanadvornykh <zandmitrij@gmail.com>
 #  Copyright 2018 Tagir Akhmetshin <tagirshin@gmail.com>
 #  This file is part of chython.
@@ -78,9 +78,10 @@ class Standardize:
         l, f = self.__standardize(double_rules)
         log.extend(l)
         fixed.update(f)
-        l, f = self.__standardize(double_rules)  # double shot rules for overlapped groups
-        log.extend(l)
-        fixed.update(f)
+        if f:
+            l, f = self.__standardize(double_rules)  # double shot rules for overlapped groups
+            log.extend(l)
+            fixed.update(f)
         l, f = self.__standardize(single_rules)
         log.extend(l)
         fixed.update(f)
@@ -344,8 +345,9 @@ class Standardize:
             bonds = self._bonds
             m = start_map
             for n in to_add:
-                m = self.add_atom(H(), _map=m)
-                bonds[n][m] = bonds[m][n] = Bond(1)
+                m = self.add_atom(H(), m)
+                bonds[n][m] = bonds[m][n] = b = Bond(1)
+                b._attach_graph(self, n, m)
                 hydrogens[n] = 0
                 log.append((n, m))
                 m += 1
@@ -424,9 +426,10 @@ class Standardize:
                                 # expected original molecule don't contain `any` bonds or these bonds not changed
                                 flush = True
                         else:
-                            bonds[n][m] = bonds[m][n] = Bond(b)
                             if b != 8:
                                 flush = True
+                            bonds[n][m] = bonds[m][n] = b = Bond(b)
+                            b._attach_graph(self, n, m)
                     log.append((tuple(match), r, str(pattern)))
 
             if not hs:  # not matched
