@@ -22,7 +22,6 @@ from functools import cached_property
 from itertools import product, chain, repeat, combinations
 from lazy_object_proxy import Proxy
 from typing import TYPE_CHECKING, Iterator, Union, List
-from weakref import ref
 from ._acid import rules as acid_rules, stripped_rules as stripped_acid_rules
 from ._base import rules as base_rules, stripped_rules as stripped_base_rules
 from ..aromatics.kekule import _kekule_component
@@ -56,14 +55,16 @@ class Tautomers:
     """
     __slots__ = ()
 
-    def neutralize(self: 'MoleculeContainer', *, logging=False, _fix_stereo=True) -> Union[bool, List[int]]:
+    def neutralize(self: 'MoleculeContainer', *, keep_charge=True, logging=False,
+                   _fix_stereo=True) -> Union[bool, List[int]]:
         """
         Convert organic salts to neutral form if possible. Only one possible form used for charge unbalanced structures.
 
-        :param logging: return changed atoms list
+        :param keep_charge: do partial neutralization to keep total charge of molecule.
+        :param logging: return changed atoms list.
         """
         try:
-            mol, changed = next(self._neutralize())
+            mol, changed = next(self._neutralize(keep_charge))
         except StopIteration:
             if logging:
                 return []
@@ -78,36 +79,6 @@ class Tautomers:
             return list(changed)
         return True
 
-    def tautomerize(self: 'MoleculeContainer', *, prepare_molecules=True, limit: int = 1000) -> bool:
-        """
-        Convert structure to canonical tautomeric form. Return True if structure changed.
-        """
-        def key(m):
-            # more aromatic rings is better
-            # less charged atoms is better
-            # lower Huckel energy is better
-            # smiles alphabetically sorted (descent). bad rule!
-            return len(m.aromatic_rings), -sum(x != 0 for x in m._charges.values()), -m.huckel_energy, str(m)
-
-        # do zwitter-ions enumeration for charged structure - search canonic ion.
-        canon = max(self.enumerate_tautomers(prepare_molecules=prepare_molecules, full=bool(int(self)), limit=limit),
-                    key=key)
-        if canon != self:  # attach state of canonic tautomer to self
-            # atoms, radicals state, parsed_mapping and plane are unchanged
-            self._bonds = canon._bonds
-            for n, m, b in self.bonds():  # move bonds attachments
-                b._Bond__graph = ref(self)
-
-            self._charges = canon._charges  # for zwitter-ionic tautomers
-            self._hydrogens = canon._hydrogens
-            self._atoms_stereo = canon._atoms_stereo
-            self._allenes_stereo = canon._allenes_stereo
-            self._cis_trans_stereo = canon._cis_trans_stereo
-            self._conformers.clear()  # flush 3d
-            self.flush_cache()
-            return True
-        return False
-
     def enumerate_tautomers(self: 'MoleculeContainer', *, prepare_molecules=True, full=True, limit: int = 1000) -> \
             Iterator['MoleculeContainer']:
         """
@@ -331,7 +302,7 @@ class Tautomers:
                     if counter == limit:
                         return
 
-    def _neutralize(self: 'MoleculeContainer'):
+    def _neutralize(self: 'MoleculeContainer', keep_charge=True):
         donors = set()
         acceptors = set()
         for q in stripped_acid_rules:
@@ -341,31 +312,41 @@ class Tautomers:
             for mapping in q.get_mapping(self, automorphism_filter=False):
                 acceptors.add(mapping[1])
 
-        if not donors or not acceptors:
-            return
-        elif len(donors) > len(acceptors):
-            copy = self.copy()
-            for a in acceptors:
-                copy._hydrogens[a] += 1
-                copy._charges[a] += 1
-            for c in combinations(donors, len(acceptors)):
-                mol = copy.copy()
-                for d in c:
+        if keep_charge:
+            if not donors or not acceptors:
+                return  # neutralization impossible
+            elif len(donors) > len(acceptors):
+                copy = self.copy()
+                for a in acceptors:
+                    copy._hydrogens[a] += 1
+                    copy._charges[a] += 1
+                for c in combinations(donors, len(acceptors)):
+                    mol = copy.copy()
+                    for d in c:
+                        mol._hydrogens[d] -= 1
+                        mol._charges[d] -= 1
+                    yield mol, acceptors.union(c)
+            elif len(donors) < len(acceptors):
+                copy= self.copy()
+                for d in donors:
+                    copy._hydrogens[d] -= 1
+                    copy._charges[d] -= 1
+                for c in combinations(acceptors, len(donors)):
+                    mol = copy.copy()
+                    for a in c:
+                        mol._hydrogens[a] += 1
+                        mol._charges[a] += 1
+                    yield mol, donors.union(c)
+            else:  # balanced!
+                mol = self.copy()
+                for d in donors:
                     mol._hydrogens[d] -= 1
                     mol._charges[d] -= 1
-                yield mol, acceptors | set(c)
-        elif len(donors) < len(acceptors):
-            copy = self.copy()
-            for d in donors:
-                copy._hydrogens[d] -= 1
-                copy._charges[d] -= 1
-            for c in combinations(acceptors, len(donors)):
-                mol = copy.copy()
-                for a in c:
+                for a in acceptors:
                     mol._hydrogens[a] += 1
                     mol._charges[a] += 1
-                yield mol, donors | set(c)
-        else:
+                yield mol, donors | acceptors
+        elif donors or acceptors:
             mol = self.copy()
             for d in donors:
                 mol._hydrogens[d] -= 1
@@ -376,6 +357,7 @@ class Tautomers:
             yield mol, donors | acceptors
 
     def _enumerate_keto_enol_tautomers(self: Union['MoleculeContainer', 'Tautomers']):
+        hs = self._hydrogens
         for fix, ket in self.__enumerate_bonds():
             if ket:
                 a = fix[-1][1]
@@ -383,6 +365,8 @@ class Tautomers:
             else:
                 a = fix[0][0]
                 d = fix[-1][1]
+            if not hs[d]:
+                continue
 
             mol = self.copy()
             m_bonds = mol._bonds
