@@ -22,7 +22,7 @@ from .functional_groups import functional_groups
 from .grid import grid_depict
 
 
-__all__ = ['functional_groups', 'grid_depict', 'fw_prepare_groups', 'fw_onehot_groups']
+__all__ = ['functional_groups', 'grid_depict', 'fw_prepare_groups']
 
 
 if find_spec('rdkit'):
